#!/usr/bin/env node
"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const fs_extra_1 = __importDefault(require("fs-extra"));
const colors_1 = __importDefault(require("colors"));
const commander_1 = __importDefault(require("commander"));
const SqlUtils_1 = __importDefault(require("./SqlUtils"));
const FSUtils_1 = __importDefault(require("./FSUtils"));
const path_1 = __importDefault(require("path"));
const generateModel_1 = __importDefault(require("./generateModel"));
const init_1 = require("./init");
require('reflect-metadata');
//设置控制台接收UTF8的字符
process.stdin.setEncoding('utf8');
//设置控制台输出的颜色
colors_1.default.setTheme({
    info: ['black', 'bold'],
    question: ['black', 'underline', 'italic'],
    warn: ['yellow', 'bold'],
    error: ['red', 'bold'],
    success: ['green', 'bold'],
    progress: ['blue']
});
commander_1.default.version('7.0.0', '-v, --version', '输出当前的版本号')
    .helpOption('-h, --help', '帮助信息')
    // .option('-m, --model <model>', '指定生成的model，不指定默认会生成所有的model以及修改config等，指定model将会只生成该model的增删改查')
    // .option('--feature <feature>', '添加功能模块')
    // .option('--fc', '查询已经集成的功能模块')
    // .option('-h, --host <host>', '主机，一般为IP地址')
    // .option('--port <port>', '端口，默认为3306')
    // .option('-u, --user <user>', '登录数据库的用户名')
    // .option('-p, --password <password>', '登录数据库的密码')
    // .option('-d, --database <database>', '需要连接的数据库')
    .option('--init <init>', '初始化一个项目')
    .option('-e, --entity', '生成数据库的所有实体类')
    .option('-t, --table <table>', '如果不指定table将会生成基本的管理系统以及实体类')
    .option('--reset', '重置生成的管理系统，会将和admin相关的都会删除，请谨慎')
    .option('--skip', '忽略依赖添加')
    .option('--to', '生成基于TypeOrm以及TypeScript的后台，该版本将直接取消admin层级的文件夹');
commander_1.default.parse(process.argv);
let reset = commander_1.default.reset;
let skip = commander_1.default.skip;
let entity = commander_1.default.entity;
// let {host, port, user, password, database, table} = program;
let table = commander_1.default.table;
let proName = commander_1.default.init;
console.log(entity);
console.log(table);
console.log(proName);
function main() {
    return __awaiter(this, void 0, void 0, function* () {
        if (proName) {
            yield init_1.init(proName);
        }
        else {
            if (!fs_extra_1.default.existsSync(process.cwd() + '/views/') || !fs_extra_1.default.existsSync(process.cwd() + '/api/')
                || !fs_extra_1.default.existsSync(process.cwd() + '/api/controllers/')
                || !fs_extra_1.default.existsSync(process.cwd() + '/assets/') || !fs_extra_1.default.existsSync(process.cwd() + '/config/')) {
                console.log(colors_1.default.error('请在项目根目录下运行'));
                return;
            }
            // const dbInfo = await getDBInfo(program);
            let dbInfo = require(path_1.default.resolve(process.cwd(), 'ormconfig.json'));
            // 覆盖数据库的重写配置，防止数据库发生问题
            dbInfo.dropSchema = false;
            dbInfo.synchronize = false;
            dbInfo.entities = undefined;
            const connection = yield SqlUtils_1.default.connect(dbInfo);
            let entities = yield SqlUtils_1.default.getEntities(connection, dbInfo.database);
            // console.log(entities);
            if (entity) {
                // 生成所有的数据库实体类
                yield SqlUtils_1.default.genaterEntities(entities);
            }
            else if (table) {
                // 生成单表的增删改查
                yield generateModel_1.default(entities, table);
            }
            else {
                FSUtils_1.default.copyTypes();
            }
            connection.close();
        }
    });
}
main();
// console.log('代码生成成功，运行后请访问 http://127.0.0.1:1337/mock/add 添加基础数据');
